@@ -25,7 +25,6 @@
   improper_ctypes,
   intra_doc_link_resolution_failure,
   late_bound_lifetime_arguments,
-  missing_copy_implementations,
   missing_debug_implementations,
   missing_docs,
   no_mangle_generic_items,
@@ -70,7 +69,6 @@ mod arg_util;
 
 mod args;
 mod commands;
-mod error;
 mod pinentry;
 #[cfg(test)]
 mod tests;
@@ -79,11 +77,6 @@ use std::env;
 use std::ffi;
 use std::io;
 use std::process;
-use std::result;
-
-use crate::error::Error;
-
-type Result<T> = result::Result<T, Error>;
 
 const NITROCLI_ADMIN_PIN: &str = "NITROCLI_ADMIN_PIN";
 const NITROCLI_USER_PIN: &str = "NITROCLI_USER_PIN";
@@ -144,7 +137,7 @@ impl<'io> Stdio for ExecCtx<'io> {
 }
 
 /// Parse the command-line arguments and execute the selected command.
-fn handle_arguments(ctx: &mut RunCtx<'_>, args: Vec<String>) -> Result<()> {
+fn handle_arguments(ctx: &mut RunCtx<'_>, args: Vec<String>) -> anyhow::Result<()> {
   use structopt::StructOpt;
 
   match args::Args::from_iter_safe(args.iter()) {
@@ -202,7 +195,7 @@ fn run<'ctx, 'io: 'ctx>(ctx: &'ctx mut RunCtx<'io>, args: Vec<String>) -> i32 {
   match handle_arguments(ctx, args) {
     Ok(()) => 0,
     Err(err) => {
-      let _ = eprintln!(ctx, "{}", err);
+      let _ = eprintln!(ctx, "{:?}", err);
       1
     }
   }
