@@ -37,7 +37,7 @@ fn mutually_exclusive_set_options() {
 }
 
 #[test_device]
-fn get(model: nitrokey::Model) -> crate::Result<()> {
+fn get(model: nitrokey::Model) -> anyhow::Result<()> {
   let re = regex::Regex::new(
     r#"^Config:
   numlock binding:          (not set|\d+)
@@ -55,8 +55,10 @@ $"#,
 
 #[test_device]
 fn set_wrong_usage(model: nitrokey::Model) {
-  let res = Nitrocli::with_model(model).handle(&["config", "set", "--numlock", "2", "-N"]);
-  let err = res.unwrap_str_err();
+  let err = Nitrocli::with_model(model)
+    .handle(&["config", "set", "--numlock", "2", "-N"])
+    .unwrap_err()
+    .to_string();
   assert!(
     err.contains("The argument '--numlock <numlock>' cannot be used with '--no-numlock'"),
     err,
@@ -64,7 +66,7 @@ fn set_wrong_usage(model: nitrokey::Model) {
 }
 
 #[test_device]
-fn set_get(model: nitrokey::Model) -> crate::Result<()> {
+fn set_get(model: nitrokey::Model) -> anyhow::Result<()> {
   let mut ncli = Nitrocli::with_model(model);
   let _ = ncli.handle(&["config", "set", "-s", "1", "-c", "0", "-N"])?;
 
