@@ -18,7 +18,6 @@
 // *************************************************************************
 
 use std::ffi;
-use std::fmt;
 
 use nitrokey_test::test as test_device;
 
@@ -35,55 +34,6 @@ mod run;
 mod status;
 mod unencrypted;
 
-/// A trait simplifying checking for expected errors.
-pub trait UnwrapError {
-  /// Unwrap an Error::Error variant.
-  fn unwrap_str_err(self) -> String;
-  /// Unwrap a Error::CommandError variant.
-  fn unwrap_cmd_err(self) -> (Option<&'static str>, nitrokey::CommandError);
-  /// Unwrap a Error::LibraryError variant.
-  fn unwrap_lib_err(self) -> (Option<&'static str>, nitrokey::LibraryError);
-}
-
-impl<T> UnwrapError for crate::Result<T>
-where
-  T: fmt::Debug,
-{
-  fn unwrap_str_err(self) -> String {
-    match self.unwrap_err() {
-      crate::Error::ClapError(err) => {
-        if err.use_stderr() {
-          err.message
-        } else {
-          String::new()
-        }
-      }
-      crate::Error::Error(err) => err,
-      err => panic!("Unexpected error variant found: {:?}", err),
-    }
-  }
-
-  fn unwrap_cmd_err(self) -> (Option<&'static str>, nitrokey::CommandError) {
-    match self.unwrap_err() {
-      crate::Error::NitrokeyError(ctx, err) => match err {
-        nitrokey::Error::CommandError(err) => (ctx, err),
-        err => panic!("Unexpected error variant found: {:?}", err),
-      },
-      err => panic!("Unexpected error variant found: {:?}", err),
-    }
-  }
-
-  fn unwrap_lib_err(self) -> (Option<&'static str>, nitrokey::LibraryError) {
-    match self.unwrap_err() {
-      crate::Error::NitrokeyError(ctx, err) => match err {
-        nitrokey::Error::LibraryError(err) => (ctx, err),
-        err => panic!("Unexpected error variant found: {:?}", err),
-      },
-      err => panic!("Unexpected error variant found: {:?}", err),
-    }
-  }
-}
-
 struct Nitrocli {
   model: Option<nitrokey::Model>,
   admin_pin: Option<ffi::OsString>,
@@ -177,7 +127,7 @@ impl Nitrocli {
   }
 
   /// Run `nitrocli`'s `handle_arguments` function.
-  pub fn handle(&mut self, args: &[&str]) -> crate::Result<String> {
+  pub fn handle(&mut self, args: &[&str]) -> anyhow::Result<String> {
     let (res, out, _) = self.do_run(args, |c, a| crate::handle_arguments(c, a));
     res.map(|_| String::from_utf8_lossy(&out).into_owned())
   }
