@@ -23,28 +23,26 @@ use crate::args;
 
 #[test_device]
 fn set_invalid_slot_raw(model: nitrokey::Model) {
-  let (rc, out, err) = Nitrocli::with_model(model).run(&["otp", "set", "100", "name", "1234"]);
+  let (rc, out, err) =
+    Nitrocli::with_model(model).run(&["otp", "set", "100", "name", "1234", "-f", "hex"]);
 
   assert_ne!(rc, 0);
   assert_eq!(out, b"");
-  assert_eq!(&err[..24], b"Could not write OTP slot");
+  assert_eq!(&err[..24], b"Failed to write OTP slot");
 }
 
 #[test_device]
 fn set_invalid_slot(model: nitrokey::Model) {
-  let res = Nitrocli::with_model(model).handle(&["otp", "set", "100", "name", "1234"]);
+  let err = Nitrocli::with_model(model)
+    .handle(&["otp", "set", "100", "name", "1234", "-f", "hex"])
+    .unwrap_err()
+    .to_string();
 
-  assert_eq!(
-    res.unwrap_lib_err(),
-    (
-      Some("Could not write OTP slot"),
-      nitrokey::LibraryError::InvalidSlot
-    )
-  );
+  assert_eq!(err, "Failed to write OTP slot");
 }
 
 #[test_device]
-fn status(model: nitrokey::Model) -> crate::Result<()> {
+fn status(model: nitrokey::Model) -> anyhow::Result<()> {
   let re = regex::Regex::new(
     r#"^alg\tslot\tname
 ((totp|hotp)\t\d+\t.+\n)+$"#,
@@ -54,7 +52,7 @@ fn status(model: nitrokey::Model) -> crate::Result<()> {
   let mut ncli = Nitrocli::with_model(model);
   // Make sure that we have at least something to display by ensuring
   // that there is one slot programmed.
-  let _ = ncli.handle(&["otp", "set", "0", "the-name", "123456"])?;
+  let _ = ncli.handle(&["otp", "set", "0", "the-name", "123456", "-f", "hex"])?;
 
   let out = ncli.handle(&["otp", "status"])?;
   assert!(re.is_match(&out), out);
@@ -62,7 +60,7 @@ fn status(model: nitrokey::Model) -> crate::Result<()> {
 }
 
 #[test_device]
-fn set_get_hotp(model: nitrokey::Model) -> crate::Result<()> {
+fn set_get_hotp(model: nitrokey::Model) -> anyhow::Result<()> {
   // Secret and expected HOTP values as per RFC 4226: Appendix D -- HOTP
   // Algorithm: Test Values.
   const SECRET: &str = "12345678901234567890";
@@ -83,7 +81,7 @@ fn set_get_hotp(model: nitrokey::Model) -> crate::Result<()> {
 }
 
 #[test_device]
-fn set_get_totp(model: nitrokey::Model) -> crate::Result<()> {
+fn set_get_totp(model: nitrokey::Model) -> anyhow::Result<()> {
   // Secret and expected TOTP values as per RFC 6238: Appendix B --
   // Test Vectors.
   const SECRET: &str = "12345678901234567890";
@@ -99,7 +97,7 @@ fn set_get_totp(model: nitrokey::Model) -> crate::Result<()> {
 }
 
 #[test_device]
-fn set_totp_uneven_chars(model: nitrokey::Model) -> crate::Result<()> {
+fn set_totp_uneven_chars(model: nitrokey::Model) -> anyhow::Result<()> {
   let secrets = [
     (args::OtpSecretFormat::Hex, "123"),
     (args::OtpSecretFormat::Base32, "FBILDWWGA2"),
@@ -113,18 +111,13 @@ fn set_totp_uneven_chars(model: nitrokey::Model) -> crate::Result<()> {
 }
 
 #[test_device]
-fn clear(model: nitrokey::Model) -> crate::Result<()> {
+fn clear(model: nitrokey::Model) -> anyhow::Result<()> {
   let mut ncli = Nitrocli::with_model(model);
   let _ = ncli.handle(&["otp", "set", "3", "hotp-test", "abcdef"])?;
   let _ = ncli.handle(&["otp", "clear", "3"])?;
   let res = ncli.handle(&["otp", "get", "3"]);
 
-  assert_eq!(
-    res.unwrap_cmd_err(),
-    (
-      Some("Could not generate OTP"),
-      nitrokey::CommandError::SlotNotProgrammed
-    )
-  );
+  let err = res.unwrap_err().to_string();
+  assert_eq!(err, "Failed to generate OTP");
   Ok(())
 }
