@@ -1,7 +1,7 @@
 // status.rs
 
 // *************************************************************************
-// * Copyright (C) 2019 Daniel Mueller (deso@posteo.net)                   *
+// * Copyright (C) 2019-2020 Daniel Mueller (deso@posteo.net)              *
 // *                                                                       *
 // * This program is free software: you can redistribute it and/or modify  *
 // * it under the terms of the GNU General Public License as published by  *
@@ -19,25 +19,29 @@
 
 use super::*;
 
-// This test acts as verification that conversion of Error::Error
-// variants into the proper exit code works properly.
 #[test_device]
 fn not_found_raw() {
   let (rc, out, err) = Nitrocli::new().run(&["status"]);
 
   assert_ne!(rc, 0);
   assert_eq!(out, b"");
-  assert_eq!(err, b"Nitrokey device not found\n");
+  let expected = r#"Nitrokey device not found
+
+Caused by:
+    Communication error: Could not connect to a Nitrokey device
+"#;
+  assert_eq!(err, expected.as_bytes());
 }
 
 #[test_device]
 fn not_found() {
   let res = Nitrocli::new().handle(&["status"]);
-  assert_eq!(res.unwrap_str_err(), "Nitrokey device not found");
+  let err = res.unwrap_err().to_string();
+  assert_eq!(err, "Nitrokey device not found");
 }
 
 #[test_device(pro)]
-fn output_pro(model: nitrokey::Model) -> crate::Result<()> {
+fn output_pro(model: nitrokey::Model) -> anyhow::Result<()> {
   let re = regex::Regex::new(
     r#"^Status:
   model:             Pro
@@ -55,7 +59,7 @@ $"#,
 }
 
 #[test_device(storage)]
-fn output_storage(model: nitrokey::Model) -> crate::Result<()> {
+fn output_storage(model: nitrokey::Model) -> anyhow::Result<()> {
   let re = regex::Regex::new(
     r#"^Status:
   model:             Storage
